from formulaparser.Parser import *
from formulaparser.syntaxtrees.AST import *

import unittest
import sys

from math import pow
from math import fabs

NUMBER=0
CELLREF=1
RANGEREF=2
FUNCTION=3

class TestVisitor(ASTVisitor):

    def __init__(self):
        self.value=[]

    def visitNumber(self, astNode):
        self.value.append(NUMBER)

    def visitCellRef(self, astNode):
        self.value.append(CELLREF)

    def visitRangeRef(self, astNode):
        self.value.append(RANGEREF)

    def visitFunction(self, astNode):
        self.value.append(FUNCTION)

class ASTSuccess(unittest.TestCase):
    """Testing AST for success
    """

    sampleNumbers = [5,4.5,3.2,-10,32,12.332,4.32,-0.002,11]

    #list of tuples (cellref, column, row, isAbsoluteColumn, isAbsoluteRow)
    sampleCellRefs = [ ["A5", stringToInt("A"), 5, 0, 0],
                       ["A55", stringToInt("A"), 55, 0, 0],
                       ["A542", stringToInt("A"), 542, 0, 0],
                       ["$A5", stringToInt("A"), 5, 1, 0],
                       ["$AZ5", stringToInt("AZ"), 5, 1, 0],
                       ["$A$5", stringToInt("A"), 5, 1, 1],
                       ["$AD5", stringToInt("AD"), 5, 1, 0],
                       ["$AB$532", stringToInt("AB"), 532, 1, 1],
                       ["FD300", stringToInt("FD"), 300, 0, 0],
                       ["A15", stringToInt("A"), 15, 0, 0],
                       ["A$143", stringToInt("A"), 143, 0, 1]
                     ]

    sampleRangeRefs = [ [CellRef(stringToInt("A"), 0, 0, 0),
                         CellRef(stringToInt("B"), 5, 0, 0),
                         [CellRef(stringToInt("A"), 0, 0, 0),
                          CellRef(stringToInt("B"), 0, 0, 0),
                          CellRef(stringToInt("A"), 1, 0, 0),
                          CellRef(stringToInt("B"), 1, 0, 0),
                          CellRef(stringToInt("A"), 2, 0, 0),
                          CellRef(stringToInt("B"), 2, 0, 0),
                          CellRef(stringToInt("A"), 3, 0, 0),
                          CellRef(stringToInt("B"), 3, 0, 0),
                          CellRef(stringToInt("A"), 4, 0, 0),
                          CellRef(stringToInt("B"), 4, 0, 0),
                          CellRef(stringToInt("A"), 5, 0, 0),
                          CellRef(stringToInt("B"), 5, 0, 0)
                         ]
                        ],
                        [CellRef(stringToInt("C"), 3, 0, 0),
                         CellRef(stringToInt("A"), 0, 0, 0),
                         [CellRef(stringToInt("C"), 3, 0, 0),
                          CellRef(stringToInt("B"), 3, 0, 0),
                          CellRef(stringToInt("A"), 3, 0, 0),
                          CellRef(stringToInt("C"), 2, 0, 0),
                          CellRef(stringToInt("B"), 2, 0, 0),
                          CellRef(stringToInt("A"), 2, 0, 0),
                          CellRef(stringToInt("C"), 1, 0, 0),
                          CellRef(stringToInt("B"), 1, 0, 0),
                          CellRef(stringToInt("A"), 1, 0, 0),
                          CellRef(stringToInt("C"), 0, 0, 0),
                          CellRef(stringToInt("B"), 0, 0, 0),
                          CellRef(stringToInt("A"), 0, 0, 0)
                         ]
                        ],
                        [CellRef(stringToInt("C"), 0, 0, 0),
                         CellRef(stringToInt("A"), 3, 0, 0),
                         [CellRef(stringToInt("C"), 0, 0, 0),
                          CellRef(stringToInt("B"), 0, 0, 0),
                          CellRef(stringToInt("A"), 0, 0, 0),
                          CellRef(stringToInt("C"), 1, 0, 0),
                          CellRef(stringToInt("B"), 1, 0, 0),
                          CellRef(stringToInt("A"), 1, 0, 0),
                          CellRef(stringToInt("C"), 2, 0, 0),
                          CellRef(stringToInt("B"), 2, 0, 0),
                          CellRef(stringToInt("A"), 2, 0, 0),
                          CellRef(stringToInt("C"), 3, 0, 0),
                          CellRef(stringToInt("B"), 3, 0, 0),
                          CellRef(stringToInt("A"), 3, 0, 0)
                         ]
                        ],
                        [CellRef(stringToInt("A"), 3, 0, 0),
                         CellRef(stringToInt("C"), 0, 0, 0),
                         [CellRef(stringToInt("A"), 3, 0, 0),
                          CellRef(stringToInt("B"), 3, 0, 0),
                          CellRef(stringToInt("C"), 3, 0, 0),
                          CellRef(stringToInt("A"), 2, 0, 0),
                          CellRef(stringToInt("B"), 2, 0, 0),
                          CellRef(stringToInt("C"), 2, 0, 0),
                          CellRef(stringToInt("A"), 1, 0, 0),
                          CellRef(stringToInt("B"), 1, 0, 0),
                          CellRef(stringToInt("C"), 1, 0, 0),
                          CellRef(stringToInt("A"), 0, 0, 0),
                          CellRef(stringToInt("B"), 0, 0, 0),
                          CellRef(stringToInt("C"), 0, 0, 0)
                         ]
                        ]
                      ]

    sampleFunctions = [ ["f",
                         [Number(5), CellRef(stringToInt("A"),5,1,1),
                          Number(10)
                         ],
                         [NUMBER,CELLREF,NUMBER,FUNCTION]
                        ],
                        ["f",
                         [Function("f",
                                   [Number(10),
                                    RangeRef(CellRef(stringToInt("A"),0,0,0),
                                             CellRef(stringToInt("A"),5,0,0))
                                   ]),
                          Number(100)
                         ],
                         [NUMBER,RANGEREF,FUNCTION,NUMBER,FUNCTION]
                        ]
                      ]


    def testStringConverter(self):
        """Testing intToString/StringToInt for success
        """
        pass
        #for i in xrange(18278):
    #        i+=1
    #        self.failIf(stringToInt(intToString(i))!=i, str(i))


    def testNumbersGetValue(self):
        """Testing Number class for success (getValue)
        """
        for k in self.sampleNumbers:
            n=Number(k)
            self.assertEqual(n.getValue(),k,
                      "getValue() method did not return the appropriate value")

    def testNumbersAccept(self):
        """Testing Number class for success (accept)
        """

        for k in self.sampleNumbers:
            n=Number(k)
            v=TestVisitor()
            n.accept(v)
            self.failIf(len(v.value)!=1)
            self.failIf(v.value[0]!=NUMBER,
                        "accept() did not behave correctly")

    def testCellRefAccessors(self):
        """Testing CellRef for success (getRow/Column)
        """

        for k in self.sampleCellRefs:
            c=CellRef(k[1],k[2],k[3],k[4])
            self.failIf(c.getColumn()!=k[1],
                        "getColumn() method returned invalid value")

            self.failIf(c.getRow()!=k[2],
                        "getRow() method returned invalid value")

    def testCellRefIsAbs(self):
        """Testing CellRef for success (isAbsRow/Column)
        """

        for k in self.sampleCellRefs:
            c=CellRef(k[1],k[2],k[3],k[4])
            self.failIf(c.isAbsColumn()!=k[3],
                        "isAbsColumn() method returned invalid value")

            self.failIf(c.isAbsRow()!=k[4],
                        "isAbsRow() method returned invalid value")

    def testCellRefAccept(self):
        """Testing CellRef for success (accept)
        """
        for k in self.sampleCellRefs:
            n=CellRef(k[1],k[2],k[3],k[4])
            v=TestVisitor()
            n.accept(v)
            self.failIf(len(v.value)!=1)
            self.failIf(v.value[0]!=CELLREF,
                        "accept() did not behave correctly")

    def testRangeRefAccessors(self):
        """Testing RangeRef for success (accessors)
        """
        for k in self.sampleCellRefs:
            for j in self.sampleCellRefs:
                c1=CellRef(k[1],k[2],k[3],k[4])
                c2=CellRef(j[1],j[2],j[3],j[4])
                r=RangeRef(c1,c2)

                c1=r.getFirstCorner()
                c2=r.getSecondCorner()
                self.failIf(c1.getColumn()!=k[1],
                            "getColumn() method returned invalid value")

                self.failIf(c1.getRow()!=k[2],
                            "getRow() method returned invalid value")
                self.failIf(c1.isAbsColumn()!=k[3],
                            "isAbsColumn() method returned invalid value")

                self.failIf(c1.isAbsRow()!=k[4],
                            "isAbsRow() method returned invalid value")

                self.failIf(c2.getColumn()!=j[1],
                            "getColumn() method returned invalid value")

                self.failIf(c2.getRow()!=j[2],
                            "getRow() method returned invalid value")
                self.failIf(c2.isAbsColumn()!=j[3],
                            "isAbsColumn() method returned invalid value")

                self.failIf(c2.isAbsRow()!=j[4],
                            "isAbsRow() method returned invalid value")

    def testRangeRefCellSet(self):
        """Testing RangeRef for success (getCellRefSet)
        """
        for sample in self.sampleRangeRefs:
            r=RangeRef(sample[0],sample[1])
            set=r.getCellRefSet()

            self.failIf(len(set)!=len(sample[2]),
                 "CellRefSet is not of expected length:"+str(len(sample[2])))

            for i in xrange(len(set)):
                self.failIf( str(set[i]) != str(sample[2][i]),
                 "An element in the returned CellRefSet is invalid")

        maxQty=0
        k=self.sampleCellRefs[0]
        for j in self.sampleCellRefs:
            c1=CellRef(k[1],k[2],k[3],k[4])
            c2=CellRef(j[1],j[2],j[3],j[4])
            r=RangeRef(c1,c2)

            c1=r.getFirstCorner()
            c2=r.getSecondCorner()

            height=abs(c1.getRow()-c2.getRow())+1
            width=abs(c1.getColumn()-c2.getColumn())+1

            self.failIf(height*width!=len(r.getCellRefSet()),
                "Length of CellRefSet is not of expected length:"+\
                str(height*width)+"!="+str(len(r.getCellRefSet())))

    def testRangeRefAccept(self):
        """Testing RangeRef for success (accept)
        """
        for k in self.sampleCellRefs:
            for j in self.sampleCellRefs:
                c1=CellRef(k[1],k[2],k[3],k[4])
                c2=CellRef(j[1],j[2],j[3],j[4])
                r=RangeRef(c1,c2)
                v=TestVisitor()
                r.accept(v)
                self.failIf(len(v.value)!=1)
                self.failIf(v.value[0]!=RANGEREF,
                            "accept() did not behave correctly")

    def testFunctionAccessorSuccess(self):
        """Testing Function for success (accessors)
        """
        for k in self.sampleFunctions:
            f=Function(k[0],k[1])

            self.failIf(f.getName()!=k[0])

            for i in xrange(len(f.getArgs())):
                self.failIf(str(k[1][i])!=str(f.getArgs()[i]))

    def testFunctionAccept(self):
        """Testing Function for success (accept)
        """
        for k in self.sampleFunctions:
            f=Function(k[0],k[1])
            v=TestVisitor()
            f.accept(v)

            for i in xrange(len(v.value)):
                self.failIf(v.value[i]!=k[2][i],
                        "accept() did not behave correctly")
































if __name__ == "__main__":
    unittest.main()







