#  ***************************************************************************
#                     Class encapsulating a SSheetData
#
#  Author: Marc Provost, 2003
#  ***************************************************************************

__DEBUG__=1

import types

class SSheetData:

    def __init__(self):
        #limits of the SSheetData
        self.__minRow=-1
        self.__maxRow=-1
        self.__minCol=-1
        self.__maxCol=-1

        #Dictionnary mapping tuple (col,row)->SSheetCell
        self.__cellData={}
        #cells in reverse topological sort for evaluation
        self.__sortedData=[]
        #Stack used to keep track of cycles when applying topo sort
        self.__stack=[]

    def getLimits(self):
        return (self.__minRow, self.__maxRow,
                self.__minCol, self.__maxCol)

    def getCell(self, col, row):
        if type(col)!=types.IntType:
            raise TypeError, "getCell expected an integer for the column"
        if type(row)!=types.IntType:
            raise TypeError, "getCell expected an integer for the row"
        if self.__cellData.has_key((col,row)):
            return self.__cellData[(col,row)][0]
        else:
            return None

    def getCellsValue(self,maxCol,maxRow):
        ans={}
        for key in self.__cellData.keys():
          if key[0]<=maxCol and key[1]<=maxRow:
            ans[key]=self.__cellData[key][0].getValue()
        return ans

    def getCellFormula(self, col, row):
        if self.__cellData.has_key((col,row)):
            return self.__cellData[(col,row)][0].getFormula()
        else:
            return ""

    def getCellValue(self, col, row):
        if self.__cellData.has_key((col,row)):
          return self.__cellData[(col,row)][0].getValue()
        else:
          return ""

    def removeCell(self, col, row):
        if type(col)!=types.IntType:
            raise TypeError, "getCell expected an integer for the column"
        if type(row)!=types.IntType:
            raise TypeError, "getCell expected an integer for the row"
        if self.__cellData.has_key((col,row)):
            del self.__cellData[(col,row)]

    def setCell(self, col, row, formulaStr):
        from SSheetCell import SSheetCell

        if (row < self.__minRow or self.__minRow==-1):
            self.__minRow = row
        if (row > self.__maxRow or self.__maxRow==-1):
            self.__maxRow = row
        if (col < self.__minCol or self.__minCol==-1):
            self.__minCol = col
        if (col > self.__maxCol or self.__maxCol==-1):
            self.__maxCol = col

        if len(formulaStr)==0:
          if self.__cellData.has_key((col,row)):
            del self.__cellData[(col,row)]
        else:
          self.__cellData[(col,row)]=[SSheetCell(col,row,formulaStr,self),1]

        self.__sortedData=[]
        self.evaluate()

    def evaluate(self):
        self.checkDependencies()
        for cell in self.__sortedData:
            cell.evaluate()

    def canEvaluate(self,cell):
        from SSheetCell import SSheetCell
        if(not isinstance(cell, SSheetCell)):
            raise TypeError,"Was expecting SSheetCell instance"
        cellKey=cell.getCoords()
        if self.__cellData.has_key(cellKey):
            return self.__cellData[cellKey][1]
        else:
            return 1

    #checkDependencies setup the cells and call __topoSort for
    #each unvisited cell.  Return a bool flag indicating cycles

    def checkDependencies(self):
        for key in self.__cellData.keys():
            cell = self.__cellData[key][0]
            self.__cellData[key][1]=1
            self.__stack=[]
            cell.visited=0
        for key in self.__cellData.keys():
            cell = self.__cellData[key][0]
            if(cell.visited==0):
                self.__topoSort(cell)

  #Traditionnal topological sort algorithm is used, the node are marked 0
  #when they are not processed, 1 when they are first saw,
  #and 2 when they are finished (i.e. no more children to
  #process).  A cycle is detected when a 1-maked cell encounder another
  #1-marked.

    def __topoSort(self, cell):
        from SSheetCell import SSheetCell
        if(not isinstance(cell, SSheetCell)):
            raise TypeError,"Was expecting SSheetCell instance"
        self.__stack.append(cell)
        cell.visited=1
        if(len(cell.getDepend())==0):
            cell.visited=2
            self.__sortedData.append(cell)
            self.__stack.pop()
        else:
            for cellRefKey in cell.getDepend():
                if self.__cellData.has_key(cellRefKey):
                    child = self.__cellData[cellRefKey][0]
                    if child.visited==0:
                        self.__topoSort(child)
                    elif child.visited==1:
                        self.__setCycle()
            cell.visited=2
            self.__stack.pop()
            self.__sortedData.append(cell)

    def __setCycle(self):
        for cell in self.__stack:
            key=cell.getCoords()
            if not self.__cellData.has_key(key):
                continue
            self.__cellData[key][1]=0

    def __str__(self):
        answer=""
        for cell in self.__sortedData:
            answer += str(cell) + '\n'
        return answer
